@@ -32,6 +32,12 @@ from handlers.base_handler import BadRequestError
 
 NetworkPort = namedtuple('NetworkPort', ['lsp', 'ls', 'dhcp_options', 'lrp'])
 Network = namedtuple('Network', ['ls', 'localnet_lsp'])
+Router = namedtuple(
+    'Router', ['lr', 'ext_gw_ls_id', 'ext_gw_dhcp_options_id', 'gw_ip']
+)
+RouterInterface = namedtuple(
+    'RouterInterface', ['id', 'ls_id', 'lsp_id', 'dhcp_options_id']
+)
 
 
 @six.add_metaclass(abc.ABCMeta)
@@ -93,6 +99,7 @@ class Mapper(object):
 
 class NetworkMapper(Mapper):
     # The names of properties received/sent in a REST request
+    REST_MTU = 'mtu'
     REST_NETWORK_ID = 'id'
     REST_NETWORK_NAME = 'name'
     REST_STATUS = 'status'
@@ -114,19 +121,22 @@ class NetworkMapper(Mapper):
             NetworkMapper.REST_PROVIDER_PHYSICAL_NETWORK)
         provider_segmentation_id = rest_network_data.get(
             NetworkMapper.REST_PROVIDER_SEGMENTATION_ID)
+        mtu = rest_network_data.get(NetworkMapper.REST_MTU)
         if network_id:
             return func(
                 wrapped_self,
                 network_id,
                 name=network_name,
                 localnet=provider_physical_network,
-                vlan=provider_segmentation_id
+                vlan=provider_segmentation_id,
+                mtu=mtu
             )
         return func(
             wrapped_self,
             name=network_name,
             localnet=provider_physical_network,
-            vlan=provider_segmentation_id
+            vlan=provider_segmentation_id,
+            mtu=mtu
         )
 
     @staticmethod
@@ -140,6 +150,9 @@ class NetworkMapper(Mapper):
             NetworkMapper.REST_TENANT_ID: tenant_id(),
             NetworkMapper.REST_STATUS: NetworkMapper.NETWORK_STATUS_ACTIVE
         }
+        mtu_value = ls.external_ids.get(ovnconst.LS_EXTERNAL_IDS_MTU)
+        if mtu_value:
+            result[NetworkMapper.REST_MTU] = int(mtu_value)
         result.update(NetworkMapper._row2rest_localnet(localnet_lsp))
         return result
 
@@ -236,6 +249,7 @@ class PortMapper(Mapper):
     OVN_DEVICE_OWNER = 'ovirt_device_owner'
     OVN_REQUESTED_CHASSIS = 'requested-chassis'
     DEVICE_OWNER_ROUTER = 'network:router_interface'
+    DEVICE_OWNER_ROUTER_GATEWAY = 'network:router_gateway'
 
     @staticmethod
     def rest2row(wrapped_self, func, rest_data, port_id):
@@ -451,8 +465,6 @@ class SubnetMapper(Mapper):
         external_ids = row.external_ids
         result = {
             SubnetMapper.REST_SUBNET_ID: str(row.uuid),
-            SubnetMapper.REST_SUBNET_NAME:
-                external_ids[SubnetMapper.OVN_NAME],
             SubnetMapper.REST_SUBNET_CIDR: row.cidr,
             SubnetMapper.REST_SUBNET_NETWORK_ID:
                 external_ids[SubnetMapper.OVN_NETWORK_ID],
@@ -466,6 +478,10 @@ class SubnetMapper(Mapper):
                 options[SubnetMapper.OVN_DNS_SERVER]
             ] if SubnetMapper.OVN_DNS_SERVER in options else []
         }
+        if SubnetMapper.OVN_NAME in external_ids:
+            result[SubnetMapper.REST_SUBNET_NAME] = (
+                external_ids[SubnetMapper.OVN_NAME]
+            )
         if SubnetMapper.OVN_GATEWAY in options:
             result[SubnetMapper.REST_SUBNET_GATEWAY_IP] = (
                 options[SubnetMapper.OVN_GATEWAY]
@@ -518,9 +534,7 @@ class RouterMapper(Mapper):
     REST_ROUTER_IP_ADDRESS = 'ip_address'
     REST_ROUTER_SUBNET_ID = 'subnet_id'
 
-    OVN_ROUTER_GATEWAY_NETWORK = 'ovirt_gateway_network'
-    OVN_ROUTER_GATEWAY_SUBNET = 'ovirt_gateway_subnet'
-    OVN_ROUTER_GATEWAY_IP = 'ovirt_gateway_ip'
+    OVN_ROUTER_GATEWAY_PORT = 'ovirt_gateway_port'
 
     ROUTER_STATUS_ACTIVE = 'ACTIVE'
     ROUTER_STATUS_INACTIVE = 'INACTIVE'
@@ -563,10 +577,10 @@ class RouterMapper(Mapper):
         return network_id, gateway_subnet, gateway_ip
 
     @staticmethod
-    def row2rest(row):
-        if not row:
+    def row2rest(router):
+        if not router:
             return {}
-
+        row = router.lr
         result = {
             RouterMapper.REST_ROUTER_ID: str(row.uuid),
             RouterMapper.REST_ROUTER_NAME: row.name,
@@ -577,39 +591,27 @@ class RouterMapper(Mapper):
                 if row.enabled else RouterMapper.ROUTER_STATUS_INACTIVE,
             RouterMapper.REST_ROUTER_ROUTES: [],
             RouterMapper.REST_TENANT_ID: tenant_id(),
+            RouterMapper.REST_ROUTER_EXTERNAL_GATEWAY_INFO:
+                RouterMapper._get_external_gateway_from_row(router),
         }
-        RouterMapper._get_external_gateway_from_row(result, row)
 
         return result
 
     @staticmethod
-    def _get_external_gateway_from_row(result, row):
-        network = row.external_ids.get(
-            RouterMapper.OVN_ROUTER_GATEWAY_NETWORK
-        )
-
-        if network:
-            gateway = {
-                RouterMapper.REST_ROUTER_NETWORK_ID: network
-            }
-            result[RouterMapper.REST_ROUTER_EXTERNAL_GATEWAY_INFO] = gateway
-
-            subnet = row.external_ids.get(
-                RouterMapper.OVN_ROUTER_GATEWAY_SUBNET
-            )
-            if subnet:
-                external_ips = {
-                    RouterMapper.REST_ROUTER_SUBNET_ID: subnet
+    def _get_external_gateway_from_row(router):
+        if not router.ext_gw_ls_id:
+            return None
+        return {
+            RouterMapper.REST_ROUTER_NETWORK_ID: router.ext_gw_ls_id,
+            RouterMapper.REST_ROUTER_ENABLE_SNAT: False,
+            RouterMapper.REST_ROUTER_FIXED_IPS: [
+                {
+                    RouterMapper.REST_ROUTER_SUBNET_ID:
+                        router.ext_gw_dhcp_options_id,
+                    RouterMapper.REST_ROUTER_IP_ADDRESS: router.gw_ip
                 }
-                gateway[RouterMapper.REST_ROUTER_FIXED_IPS] = [external_ips]
-
-                ip = row.external_ids.get(
-                    RouterMapper.OVN_ROUTER_GATEWAY_IP
-                )
-                if ip:
-                    external_ips[RouterMapper.REST_ROUTER_IP_ADDRESS] = ip
-        else:
-            result[RouterMapper.REST_ROUTER_EXTERNAL_GATEWAY_INFO] = None
+            ]
+        }
 
     @staticmethod
     def validate_add_rest_input(rest_data):
@@ -681,6 +683,24 @@ class BaseRouterInterfaceMapper(Mapper):
             wrapped_self, router_id=router_id, subnet_id=subnet, port_id=port
         )
 
+    @staticmethod
+    def row2rest(row):
+        if not row:
+            return {}
+
+        return {
+            AddRouterInterfaceMapper.REST_ROUTERINTERFACE_ID: row.id,
+            AddRouterInterfaceMapper.REST_ROUTERINTERFACE_NETWORK_ID:
+                row.ls_id,
+            AddRouterInterfaceMapper.REST_ROUTERINTERFACE_PORT_ID: row.lsp_id,
+            AddRouterInterfaceMapper.REST_ROUTERINTERFACE_SUBNET_ID:
+                row.dhcp_options_id,
+            AddRouterInterfaceMapper.REST_ROUTERINTERFACE_SUBNET_IDS: [
+                row.dhcp_options_id
+            ],
+            AddRouterInterfaceMapper.REST_TENANT_ID: tenant_id(),
+        }
+
     @staticmethod
     def validate_add_rest_input(rest_data):
         raise MethodNotAllowedError(
@@ -707,27 +727,6 @@ class BaseRouterInterfaceMapper(Mapper):
 
 
 class AddRouterInterfaceMapper(BaseRouterInterfaceMapper):
-
-    @staticmethod
-    def row2rest(row):
-        if not row:
-            return {}
-
-        router_id, network_id, port_id, subnet_id = row
-
-        return {
-            AddRouterInterfaceMapper.REST_ROUTERINTERFACE_ID: router_id,
-            AddRouterInterfaceMapper.REST_ROUTERINTERFACE_NETWORK_ID:
-                network_id,
-            AddRouterInterfaceMapper.REST_ROUTERINTERFACE_PORT_ID: port_id,
-            AddRouterInterfaceMapper.REST_ROUTERINTERFACE_SUBNET_ID:
-                subnet_id,
-            AddRouterInterfaceMapper.REST_ROUTERINTERFACE_SUBNET_IDS: [
-                subnet_id
-            ],
-            AddRouterInterfaceMapper.REST_TENANT_ID: tenant_id(),
-        }
-
     @staticmethod
     def validate_update_rest_input(rest_data):
         subnet, port = BaseRouterInterfaceMapper._validate_update(rest_data)
@@ -738,7 +737,6 @@ class AddRouterInterfaceMapper(BaseRouterInterfaceMapper):
 
 
 class RemoveRouterInterfaceMapper(BaseRouterInterfaceMapper):
-
     @staticmethod
     def validate_update_rest_input(rest_data):
         BaseRouterInterfaceMapper._validate_update(rest_data)
