@@ -20,7 +20,6 @@ from __future__ import absolute_import
 from uuid import UUID
 import mock
 import pytest
-import six
 
 import ovndb.constants as ovnconst
 
@@ -30,12 +29,15 @@ from ovirt_provider_config_common import dhcp_mtu
 from ovirt_provider_config_common import dhcp_server_mac
 from ovirt_provider_config_common import tenant_id
 from ovndb.ovn_north import OvnNorth
+from ovndb.ovn_north_mappers import Network
 from ovndb.ovn_north_mappers import NetworkMapper
 from ovndb.ovn_north_mappers import PortMapper
 from ovndb.ovn_north_mappers import SubnetConfigError
 from ovndb.ovn_north_mappers import SubnetMapper
 from ovndb.ovn_north_mappers import UnsupportedDataValueError
 
+from ovntestlib import assert_network_equal
+from ovntestlib import assert_subnet_equal
 from ovntestlib import OvnNetworkRow
 from ovntestlib import OvnPortRow
 from ovntestlib import OvnRouterPort
@@ -53,13 +55,16 @@ class TestOvnNorth(object):
     LOCALNET_VLAN = 10
     DEVICE_OWNER_OVIRT = 'oVirt'
     SUBNET_CIDR = '1.1.1.0/24'
+    VALUE_NETWORK_MTU = 32854
 
     NETWORK_ID10 = UUID(int=10)
     NETWORK_ID11 = UUID(int=11)
     NETWORK_ID12 = UUID(int=12)
+    NETWORK_IDMTU = UUID(int=123)
     NETWORK_NAME10 = 'name10'
     NETWORK_NAME11 = 'name11'
     NETWORK_NAME12 = 'name12'
+    NETWORK_NAMEMTU = 'nameMTU'
 
     PORT_ID01 = UUID(int=1)
     PORT_ID02 = UUID(int=2)
@@ -107,6 +112,7 @@ class TestOvnNorth(object):
 
     SUBNET_ID101 = UUID(int=101)
     SUBNET_ID102 = UUID(int=102)
+    SUBNET_IDMTU = UUID(int=987)
 
     SUBNET_101 = OvnSubnetRow(
         SUBNET_ID101,
@@ -116,6 +122,12 @@ class TestOvnNorth(object):
 
     SUBNET_102 = OvnSubnetRow(SUBNET_ID102, cidr=SUBNET_CIDR)
 
+    SUBNET_MTU = OvnSubnetRow(
+        SUBNET_IDMTU,
+        network_id=str(NETWORK_IDMTU),
+        cidr=SUBNET_CIDR
+    )
+
     NETWORK_10 = OvnNetworkRow(NETWORK_ID10, NETWORK_NAME10)
     NETWORK_11 = OvnNetworkRow(
         NETWORK_ID11, NETWORK_NAME11, ports=[PORT_1, PORT_2]
@@ -129,6 +141,10 @@ class TestOvnNorth(object):
         NETWORK_NAME12,
         ports=[PORT_3]
     )
+    NETWORK_MTU = OvnNetworkRow(
+        NETWORK_IDMTU, NETWORK_NAMEMTU,
+        external_ids={NetworkMapper.REST_MTU: str(VALUE_NETWORK_MTU)}
+    )
 
     ROUTER_ID20 = UUID(int=20)
     ROUTER_NAME20 = 'router20'
@@ -141,47 +157,6 @@ class TestOvnNorth(object):
 
     subnets = [SUBNET_101, SUBNET_102]
 
-    def assert_networks_equal(self, actual, network_row):
-        assert actual['id'] == str(network_row.uuid)
-        assert actual['name'] == network_row.name
-        assert actual['tenant_id'] == tenant_id()
-        self._assert_localnet_networks_equal(actual, network_row)
-
-    def _assert_localnet_networks_equal(self, actual, net_row):
-        actual_localnet_conf = self._get_localnet_conf_from_net_rest(actual)
-        expected_localnet_conf = self._get_localnet_conf_from_net_row(net_row)
-        assert actual_localnet_conf == expected_localnet_conf
-
-    def _get_localnet_conf_from_net_rest(self, network_rest):
-        return {
-            key: val for key, val in six.viewitems(network_rest)
-            if key in (
-                NetworkMapper.REST_PROVIDER_PHYSICAL_NETWORK,
-                NetworkMapper.REST_PROVIDER_SEGMENTATION_ID,
-                NetworkMapper.REST_PROVIDER_NETWORK_TYPE
-            )
-        }
-
-    def _get_localnet_conf_from_net_row(self, net_row):
-        if not net_row.ports:
-            return {}
-        conf = {}
-        localnet_port = net_row.ports[0]
-        physical_net = localnet_port.options.get(
-            ovnconst.LSP_OPTION_NETWORK_NAME
-        )
-        segmentation_id = localnet_port.tag[0] if localnet_port.tag else None
-        if physical_net and segmentation_id:
-            conf[NetworkMapper.REST_PROVIDER_NETWORK_TYPE] = \
-                NetworkMapper.NETWORK_TYPE_VLAN
-            conf[NetworkMapper.REST_PROVIDER_PHYSICAL_NETWORK] = physical_net
-            conf[NetworkMapper.REST_PROVIDER_SEGMENTATION_ID] = segmentation_id
-        elif physical_net:
-            conf[NetworkMapper.REST_PROVIDER_NETWORK_TYPE] = \
-                NetworkMapper.NETWORK_TYPE_FLAT
-            conf[NetworkMapper.REST_PROVIDER_PHYSICAL_NETWORK] = physical_net
-        return conf
-
     def assert_port_equal(self, actual, port_row, network_id):
         assert actual['id'] == str(port_row.uuid)
         assert actual['network_id'] == network_id
@@ -204,9 +179,19 @@ class TestOvnNorth(object):
         result = ovn_north.list_networks()
 
         assert len(result) == 3
-        self.assert_networks_equal(result[0], TestOvnNorth.NETWORK_10)
-        self.assert_networks_equal(result[1], TestOvnNorth.NETWORK_11)
-        self.assert_networks_equal(result[2], TestOvnNorth.NETWORK_LOCALNET_12)
+        assert_network_equal(
+            result[0], Network(ls=TestOvnNorth.NETWORK_10, localnet_lsp=None)
+        )
+        assert_network_equal(
+            result[1], Network(ls=TestOvnNorth.NETWORK_11, localnet_lsp=None)
+        )
+        localnet_network = TestOvnNorth.NETWORK_LOCALNET_12
+        assert_network_equal(
+            result[2],
+            Network(
+                ls=localnet_network, localnet_lsp=localnet_network.ports[0]
+            )
+        )
         assert mock_ls_list.call_count == 1
         assert mock_ls_list.return_value.execute.call_count == 1
 
@@ -221,7 +206,8 @@ class TestOvnNorth(object):
         ovn_north = OvnNorth()
         result = ovn_north.get_network(str(TestOvnNorth.NETWORK_ID10))
 
-        self.assert_networks_equal(result, TestOvnNorth.NETWORK_10)
+        network = Network(ls=TestOvnNorth.NETWORK_10, localnet_lsp=None)
+        assert_network_equal(result, network)
         assert mock_ls_get.call_count == 1
         assert mock_ls_get.return_value.execute.call_count == 1
         assert mock_ls_get.mock_calls[0] == mock.call(
@@ -242,12 +228,14 @@ class TestOvnNorth(object):
         }
         result = ovn_north.add_network(rest_data)
 
-        self.assert_networks_equal(result, TestOvnNorth.NETWORK_10)
+        network = Network(ls=TestOvnNorth.NETWORK_10, localnet_lsp=None)
+        assert_network_equal(result, network)
         assert mock_add_command.call_count == 1
         assert mock_add_command.mock_calls[0] == mock.call(
             ovn_north.idl,
             TestOvnNorth.NETWORK_NAME10,
-            False
+            False,
+            external_ids={}
         )
 
     @mock.patch('ovsdbapp.schema.ovn_northbound.commands.LsAddCommand')
@@ -273,12 +261,19 @@ class TestOvnNorth(object):
         }
         result = ovn_north.add_network(rest_data)
 
-        self.assert_networks_equal(result, TestOvnNorth.NETWORK_LOCALNET_12)
+        localnet_network = TestOvnNorth.NETWORK_LOCALNET_12
+        assert_network_equal(
+            result,
+            Network(
+                ls=localnet_network, localnet_lsp=localnet_network.ports[0]
+            )
+        )
         assert mock_ls_add_command.call_count == 1
         assert mock_ls_add_command.mock_calls[0] == mock.call(
             ovn_north.idl,
             TestOvnNorth.NETWORK_NAME12,
-            False
+            False,
+            external_ids={}
         )
         assert mock_lsp_add_command.call_count == 1
         assert mock_lsp_add_command.mock_calls[0] == mock.call(
@@ -292,6 +287,177 @@ class TestOvnNorth(object):
         assert mock_db_set_command.call_count == 2
         assert mock_ls_get_command.call_count == 1
 
+    @mock.patch(
+        'ovsdbapp.schema.ovn_northbound.commands.DhcpOptionsGetCommand.'
+        'execute',
+        lambda cmd, check_error: TestOvnNorth.SUBNET_MTU
+    )
+    @mock.patch(
+        'ovsdbapp.schema.ovn_northbound.commands.DhcpOptionsAddCommand.'
+        'execute',
+        lambda cmd, check_error: TestOvnNorth.SUBNET_MTU
+    )
+    @mock.patch(
+        'ovsdbapp.schema.ovn_northbound.commands.DhcpOptionsListCommand.'
+        'execute',
+        lambda cmd, check_error: []
+    )
+    @mock.patch(
+        'ovsdbapp.schema.ovn_northbound.commands.LsAddCommand.execute',
+        lambda cmd, check_error: TestOvnNorth.NETWORK_MTU
+    )
+    @mock.patch(
+        'ovsdbapp.schema.ovn_northbound.commands.LsGetCommand.execute',
+        lambda cmd, check_error: TestOvnNorth.NETWORK_MTU
+    )
+    @mock.patch(
+        'ovsdbapp.backend.ovs_idl.command.DbSetCommand',
+        autospec=False
+    )
+    @mock.patch(
+        'ovsdbapp.schema.ovn_northbound.commands.DhcpOptionsSetOptionsCommand',
+        autospec=False
+    )
+    def test_add_network_with_mtu(
+            self,
+            mock_setoptions_command,
+            mock_dbset_command,
+            mock_connection
+    ):
+        ovn_north = OvnNorth()
+        network_rest_data = {
+            NetworkMapper.REST_NETWORK_NAME: TestOvnNorth.NETWORK_NAMEMTU,
+            NetworkMapper.REST_MTU: TestOvnNorth.VALUE_NETWORK_MTU
+        }
+        network_creation_result = ovn_north.add_network(network_rest_data)
+        assert_network_equal(
+            network_creation_result,
+            Network(ls=TestOvnNorth.NETWORK_MTU, localnet_lsp=None)
+        )
+
+        # create a subnet associated with the above network
+        subnet_rest_data = {
+            SubnetMapper.REST_SUBNET_NAME:
+                TestOvnNorth.SUBNET_102.external_ids.get(
+                    SubnetMapper.OVN_NAME
+                ),
+            SubnetMapper.REST_SUBNET_CIDR: TestOvnNorth.SUBNET_CIDR,
+            SubnetMapper.REST_SUBNET_NETWORK_ID:
+                str(TestOvnNorth.NETWORK_IDMTU),
+            SubnetMapper.REST_SUBNET_DNS_NAMESERVERS: ['1.1.1.1'],
+            SubnetMapper.REST_SUBNET_GATEWAY_IP: '1.1.1.0',
+        }
+
+        expected_options_call = mock.call(
+            ovn_north.idl,
+            TestOvnNorth.SUBNET_IDMTU,
+            dns_server='1.1.1.1',
+            lease_time=dhcp_lease_time(),
+            router='1.1.1.0',
+            server_id='1.1.1.0',
+            server_mac=dhcp_server_mac(),
+            mtu=str(TestOvnNorth.VALUE_NETWORK_MTU)
+        )
+
+        subnet_creation_result = ovn_north.add_subnet(subnet_rest_data)
+        assert mock_setoptions_command.call_count == 1
+        assert mock_setoptions_command.mock_calls[0] == expected_options_call
+        assert mock_dbset_command.call_count == 1
+        assert_subnet_equal(subnet_creation_result, TestOvnNorth.SUBNET_MTU)
+
+    @mock.patch(
+        'ovsdbapp.schema.ovn_northbound.commands.DhcpOptionsGetCommand.'
+        'execute',
+        lambda cmd, check_error: TestOvnNorth.SUBNET_MTU
+    )
+    @mock.patch(
+        'ovsdbapp.schema.ovn_northbound.commands.DhcpOptionsAddCommand.'
+        'execute',
+        lambda cmd, check_error: TestOvnNorth.SUBNET_MTU
+    )
+    @mock.patch(
+        'ovsdbapp.schema.ovn_northbound.commands.LsAddCommand.execute',
+        lambda cmd, check_error: TestOvnNorth.NETWORK_MTU
+    )
+    @mock.patch(
+        'ovsdbapp.schema.ovn_northbound.commands.LsGetCommand.execute',
+        lambda cmd, check_error: TestOvnNorth.NETWORK_MTU
+    )
+    @mock.patch(
+        'ovsdbapp.backend.ovs_idl.command.DbSetCommand',
+        autospec=False
+    )
+    @mock.patch(
+        'ovsdbapp.schema.ovn_northbound.commands.DhcpOptionsSetOptionsCommand',
+        autospec=False
+    )
+    @mock.patch(
+        'ovsdbapp.schema.ovn_northbound.commands.DhcpOptionsListCommand'
+    )
+    def test_update_networks_mtu(
+            self,
+            mock_dhcp_list_command,
+            mock_setoptions_command,
+            mock_dbset_command,
+            mock_connection
+    ):
+        mock_dhcp_list_command.return_value.execute.return_value = []
+
+        ovn_north = OvnNorth()
+        network_rest_data = {
+            NetworkMapper.REST_NETWORK_NAME: TestOvnNorth.NETWORK_NAMEMTU,
+            NetworkMapper.REST_MTU: TestOvnNorth.VALUE_NETWORK_MTU
+        }
+        ovn_north.add_network(network_rest_data)
+
+        # create a subnet associated with the above network
+        subnet_rest_data = {
+            SubnetMapper.REST_SUBNET_NAME:
+                TestOvnNorth.SUBNET_102.external_ids.get(
+                    SubnetMapper.OVN_NAME
+                ),
+            SubnetMapper.REST_SUBNET_CIDR: TestOvnNorth.SUBNET_CIDR,
+            SubnetMapper.REST_SUBNET_NETWORK_ID:
+                str(TestOvnNorth.SUBNET_IDMTU),
+            SubnetMapper.REST_SUBNET_DNS_NAMESERVERS: ['1.1.1.1'],
+            SubnetMapper.REST_SUBNET_GATEWAY_IP: '1.1.1.0',
+        }
+
+        ovn_north.add_subnet(subnet_rest_data)
+        new_mtu = 14999
+        mtu_update = {
+            NetworkMapper.REST_NETWORK_NAME: TestOvnNorth.NETWORK_NAMEMTU,
+            NetworkMapper.REST_MTU: new_mtu
+        }
+
+        assert mock_setoptions_command.call_count == 1
+        mock_dhcp_list_command.return_value.execute.return_value = [
+            TestOvnNorth.SUBNET_MTU
+        ]
+
+        ovn_north.update_network(mtu_update, TestOvnNorth.NETWORK_IDMTU)
+        expected_external_ids_update = mock.call(
+            ovn_north.idl,
+            ovnconst.TABLE_LS,
+            TestOvnNorth.NETWORK_IDMTU,
+            (ovnconst.ROW_LSP_NAME, TestOvnNorth.NETWORK_NAMEMTU),
+            (
+                ovnconst.ROW_LSP_EXTERNAL_IDS,
+                {NetworkMapper.REST_MTU: str(new_mtu)},
+            ),
+        )
+        expected_network_mtu_update = mock.call(
+            ovn_north.idl,
+            ovnconst.TABLE_DHCP_Options,
+            TestOvnNorth.SUBNET_IDMTU,
+            (
+                ovnconst.ROW_DHCP_OPTIONS,
+                {SubnetMapper.OVN_DHCP_MTU: str(new_mtu)}
+            )
+        )
+        assert expected_external_ids_update in mock_dbset_command.mock_calls
+        assert expected_network_mtu_update in mock_dbset_command.mock_calls
+
     @mock.patch(
         'ovsdbapp.schema.ovn_northbound.commands.LsGetCommand.execute',
         lambda cmd, check_error: OvnNetworkRow(
@@ -310,7 +476,8 @@ class TestOvnNorth(object):
         }
         result = ovn_north.update_network(rest_data, TestOvnNorth.NETWORK_ID10)
 
-        self.assert_networks_equal(result, TestOvnNorth.NETWORK_10)
+        network = Network(ls=TestOvnNorth.NETWORK_10, localnet_lsp=None)
+        assert_network_equal(result, network)
         assert mock_set_command.call_count == 1
         assert mock_set_command.mock_calls[0] == mock.call(
             ovn_north.idl,
@@ -339,7 +506,13 @@ class TestOvnNorth(object):
         }
         result = ovn_north.update_network(rest_data, TestOvnNorth.NETWORK_ID12)
 
-        self.assert_networks_equal(result, TestOvnNorth.NETWORK_LOCALNET_12)
+        localnet_network = TestOvnNorth.NETWORK_LOCALNET_12
+        assert_network_equal(
+            result,
+            Network(
+                ls=localnet_network, localnet_lsp=localnet_network.ports[0]
+            )
+        )
         assert mock_set_command.call_count == 2
         assert mock_set_command.mock_calls[0] == mock.call(
             ovn_north.idl,
@@ -630,7 +803,10 @@ class TestOvnNorth(object):
         add_execute.return_value = TestOvnNorth.SUBNET_102
         ovn_north = OvnNorth()
         rest_data = {
-            SubnetMapper.REST_SUBNET_NAME: 'subnet_name',
+            SubnetMapper.REST_SUBNET_NAME:
+                TestOvnNorth.SUBNET_102.external_ids.get(
+                    SubnetMapper.OVN_NAME
+                ),
             SubnetMapper.REST_SUBNET_CIDR: TestOvnNorth.SUBNET_CIDR,
             SubnetMapper.REST_SUBNET_NETWORK_ID:
                 str(TestOvnNorth.NETWORK_ID10),
@@ -638,7 +814,7 @@ class TestOvnNorth(object):
             SubnetMapper.REST_SUBNET_GATEWAY_IP: '1.1.1.0',
         }
         result = ovn_north.add_subnet(rest_data)
-        assert result['id'] == str(TestOvnNorth.SUBNET_ID102)
+        assert_subnet_equal(result, TestOvnNorth.SUBNET_102)
         assert mock_dbset_command.call_count == 1
         assert mock_add_command.call_count == 1
         assert mock_setoptions_command.call_count == 1
@@ -657,7 +833,9 @@ class TestOvnNorth(object):
         expected_add_call = mock.call(
             ovn_north.idl,
             TestOvnNorth.SUBNET_CIDR,
-            ovirt_name='subnet_name',
+            ovirt_name=TestOvnNorth.SUBNET_102.external_ids.get(
+                    SubnetMapper.OVN_NAME
+                ),
             ovirt_network_id=str(TestOvnNorth.NETWORK_ID10)
         )
         assert mock_add_command.mock_calls[0] == expected_add_call
@@ -714,7 +892,7 @@ class TestOvnNorth(object):
             SubnetMapper.REST_SUBNET_GATEWAY_IP: '1.1.1.0',
         }
         result = ovn_north.add_subnet(rest_data)
-        assert result['id'] == str(TestOvnNorth.SUBNET_ID102)
+        assert_subnet_equal(result, TestOvnNorth.SUBNET_102)
         assert mock_dbset_command.call_count == 1
         assert mock_add_command.call_count == 1
         assert mock_setoptions_command.call_count == 1
